#!/usr/bin/env python

# At the moment, this extracts various line segments from an SVG file.
# The id="..." fields must be careful set in Inkscape.
# Dimensions in Inkscape are in mm (so set the number of 'px' pixels to be the size in metres/1000)
# The line segments are then extruded based on the hard-coded lengths in this file
# The line segments are then mirrored across the X-axis and extruded again, giving the whole body shell

import xml.dom.minidom

p = xml.dom.minidom.parse('class373-profile.svg')

def extractColour(node):
   c = dict([css.split(':') for css in str(node.getAttribute('style')).split(';')])['stroke'].lstrip('#')
   if len(c) < 6:
       c = ''.join([''.join(a) for a in zip(c,c)])
   return tuple([int(i,16) for i in c[0:2],c[2:4],c[4:6]])

def extractVerts(node, scale=0.001):
    return [[n * scale for n in map(int,map(float,v.split(',')))] for v in str(node.getAttribute('d')).split(' ')[1::2]]

class ObjectCsv:
    def __init__(self):
        self.s = ''
    def face(self,vs,colour=None,anticlockwise=False):
        self.s += 'CreateMeshBuilder\n'
        for v in vs:
            self.s += 'AddVertex,%g,%g,%g\n' % v
        faces = range(len(vs))
        if anticlockwise:
            faces.reverse()
        self.s += 'AddFace2,'+','.join(map(str,faces))+'\n'
        if colour:
            self.s += 'SetColor,%d,%d,%d\n' % colour
        self.s += 'GenerateNormals\n'
    def comment(self, message=None):
        if message is not None:
            message = message.replace('\n', '\n; ')
            self.s += '; ' + message + '\n'
        else:
            self.s += '\n'
    def dump(self):
       return self.s
    def bogie(self,position=6,width=4,height=1,gauge=1.5,texture='class373-loco-bogie-green.png'):
       self.s += '''
; bogie @ %(position)
CreateMeshBuilder
AddVertex %(gauge),0,%(posa)
AddVertex %(gauge),%(height),%(posa)
AddVertex %(gauge),%(height),%(posb)
AddVertex %(gauge),0,%(posb)
AddFace 0,1,2,3
SetColor 100,100,100
GenerateNormals
LoadTexture,%(texture),
SetTextureCoordinates,0, 0, 1,
SetTextureCoordinates,1, 0, 0,
SetTextureCoordinates,2, 1, 0,
SetTextureCoordinates,3, 1, 1,
''' % {
          'position': position,
          'gauge': gauge,
          'posa': position-(0.5*width),
          'posb': position+(0.5*width),
          'texture': texture}

class Line:
    def __repr__(self):
        return `self.id` + `self.colour` + `self.vs`
    def extrude_and_render(self,front=-10,back=10,mirror=False,otherway=False,format=ObjectCsv):
        o = format()
        for a,b in zip(self.vs[:-1],self.vs[1:]):
            scalex = 1.0
            if mirror:
                scalex = -1.0
            o.comment(self.id)
            o.face([(scalex*a[0],a[1],front),
                  (scalex*b[0],b[1],front),
                  (scalex*b[0],b[1],back),
                  (scalex*a[0],a[1],back)],
                 self.colour,anticlockwise=(mirror^otherway))
        print o.dump()

lines = []

for i in p.getElementsByTagName('path'):
    l = Line()
    l.id = str(i.getAttribute('id'))
    l.vs = extractVerts(i)
    l.colour = extractColour(i)
    lines.append(l)

front = -9.0
back = 9.0

wrong_way = []
ignore_way = ['roof-opposite-top-dont-render']
#ignore_way += ['door-skin-inner', 'door-skin-outer', 'walkway']

if True:
    # End car
    LENGTH = 21.84
    HALF = LENGTH*0.5
    CORRIDOR = 0.8
    BULKHEAD = HALF-CORRIDOR
    OVERHANG = BULKHEAD + 0.3
    ROOF = BULKHEAD + 0.1
    AROOF = OVERHANG - 0.5
    SKIRT = OVERHANG - 0.1
    UNDERFRAME = HALF - 1.5

    extrude_lengths = {
        'door-skin-inner': [HALF,BULKHEAD],
        'door-skin-outer': [HALF,BULKHEAD],
        'walkway': [HALF,BULKHEAD],

        'roof-top': [ROOF,-AROOF],
        'roof-middle': [ROOF,-OVERHANG],

        'roof-edge': [OVERHANG,-OVERHANG],
        'side': [OVERHANG,-OVERHANG],
        'side-curved': [OVERHANG,-OVERHANG],
        'skirt': [OVERHANG,-SKIRT],
        'underfloor': [OVERHANG,-UNDERFRAME]
        }
else:
    # Centre car
    ignore_way += ['door-skin-inner', 'door-skin-outer', 'walkway']

    LENGTH = 18.70
    HALF = LENGTH*0.5
    CORRIDOR = 0.8
    BULKHEAD = HALF-0.6
    OVERHANG = HALF-0.1
    ROOF = BULKHEAD + 0.1
    SKIRT = OVERHANG - 0.1
    UNDERFRAME = HALF - 1.5

    extrude_lengths = {
        'roof-top': [ROOF,-ROOF],
        'roof-middle': [OVERHANG,-OVERHANG],

        'roof-edge': [OVERHANG,-OVERHANG],
        'side': [OVERHANG,-OVERHANG],
        'side-curved': [OVERHANG,-OVERHANG],
        'skirt': [SKIRT,-SKIRT],
        'underfloor': [UNDERFRAME,-UNDERFRAME]
        }

for l in lines:
    if l.id in wrong_way: l.vs.reverse()

def fixup(x,y):
    if abs(x) < 0.05:
        x = 0
    return x,3.334-y

for l in lines:
    for i in range(len(l.vs)):
        l.vs[i] = fixup(*l.vs[i])

for l in lines:
    if l.id in ignore_way:
        continue
    else:
        if True:
            start, end = extrude_lengths[l.id]
            l.extrude_and_render(start,end,otherway=True)
            l.extrude_and_render(start,end,mirror=True,otherway=True)
        else:
            start, end = extrude_lengths[l.id]
            start, end = -end, -start
            l.extrude_and_render(start,end,otherway=True)
            l.extrude_and_render(start,end,mirror=True,otherway=True)
